@@ -1,5 +1,5 @@
 package Job::Machine::Base;
-$Job::Machine::Base::VERSION = '0.22';
+$Job::Machine::Base::VERSION = '0.23';
 use strict;
 use warnings;
 use Carp;
@@ -26,7 +26,7 @@ sub subscribe {
 	return $self->db->listen(queue => $queue, reply => $reply);
 }
 
-sub log {
+sub job_log {
 	my ($self, $msg) = @_;
 	print STDERR $msg, "\n";
 	return;
@@ -48,7 +48,7 @@ Job::Machine::Base
 
 =head1 VERSION
 
-version 0.22
+version 0.23
 
 =head1 NAME
 
@@ -88,7 +88,7 @@ to open a database.
 	timeout is how long to wait for notifications before doing a housekeeping loop.
 	Default is 5 minutes.
 
-=head2 log
+=head2 job_log
 
 Give it a text and it will log it.
 
