@@ -1,5 +1,5 @@
 package Job::Machine::DB;
-$Job::Machine::DB::VERSION = '0.22';
+$Job::Machine::DB::VERSION = '0.23';
 use strict;
 use warnings;
 use Carp qw/croak confess/;
@@ -9,6 +9,7 @@ use Data::Serializer;
 use constant QUEUE_PREFIX    => 'jm:';
 use constant RESPONSE_PREFIX => 'jmr:';
 
+
 sub new {
 	my ($class, %args) = @_;
 	croak "No connect information" unless $args{dbh} or $args{dsn};
@@ -23,6 +24,7 @@ sub new {
 	return bless \%args, $class;
 }
 
+
 sub serializer {
 	my ($self) = @_;
 	my $args = $self->{serializer_args} || {};
@@ -30,6 +32,7 @@ sub serializer {
 	return $self->{serialize} ||= Data::Serializer->new(%$args);
 }
 
+
 sub listen {
 	my ($self, %args) = @_;
 	my $queue = $args{queue} || return undef;
@@ -40,6 +43,7 @@ sub listen {
 	}
 }
 
+
 sub unlisten {
 	my ($self, %args) = @_;
 	my $queue = $args{queue} || return undef;
@@ -50,6 +54,7 @@ sub unlisten {
 	}
 }
 
+
 sub notify {
 	my ($self, %args) = @_;
 	my $queue = $args{queue} || return undef;
@@ -63,6 +68,7 @@ sub notify {
 	);
 }
 
+
 sub get_notification {
 	my ($self,$timeout) = @_;
 	my $dbh = $self->dbh;
@@ -70,6 +76,7 @@ sub get_notification {
 	return $notifies;
 }
 
+
 sub set_listen {
 	my ($self,$timeout) = @_;
 	my $dbh = $self->dbh;
@@ -83,8 +90,9 @@ sub set_listen {
 	return $notifies || [0,0];
 }
 
+
 sub fetch_work_task {
-	my ($self,$pid) = @_;
+	my $self = shift;
 	my $queue = ref $self->{queue} ? $self->{queue} : [$self->{queue}];
 	$self->{current_table} = 'task';
 	my $elems = join(',', ('?') x @$queue);
@@ -117,6 +125,7 @@ sub fetch_work_task {
 	return $task;
 }
 
+
 sub insert_task {
 	my ($self,$data,$queue) = @_;
 	my $class = $self->fetch_class($queue);
@@ -131,6 +140,7 @@ sub insert_task {
 	$self->insert(sql => $sql,data => [$class->{class_id},$frozen,0]);
 }
 
+
 sub set_task_status {
 	my ($self,$status) = @_;
 	my $id = $self->task_id;
@@ -143,6 +153,7 @@ sub set_task_status {
 	$self->update(sql => $sql,data => [$status,$id]);
 }
 
+
 sub fetch_class {
 	my ($self,$queue) = @_;
 	$self->{current_table} = 'class';
@@ -154,6 +165,24 @@ sub fetch_class {
 	return $self->select_first(sql => $sql,data => [$queue]) || $self->insert_class($queue);
 }
 
+
+sub fetch_task {
+	my ($self,$id) = @_;
+	$self->{current_table} = 'task';
+	my $sql = qq{
+		SELECT *
+		FROM "$self->{database_schema}".$self->{current_table}
+		JOIN "$self->{database_schema}".class USING (class_id)
+		WHERE task_id=?
+	};
+	my $task = $self->select_first(sql => $sql,data => [$id]) or return;
+
+	$task->{frozen} = $task->{parameters};
+	$task->{parameters} = $self->serializer->deserialize($task->{parameters});
+	return $task;
+}
+
+
 sub insert_class {
 	my ($self,$queue) = @_;
 	my $sql = qq{
@@ -165,21 +194,49 @@ sub insert_class {
 	$self->select_first(sql => $sql,data => [$queue]);
 }
 
+
 sub insert_result {
-	my ($self,$data,$queue) = @_;
+	my ($self,$data) = @_;
 	$self->{current_table} = 'result';
-	my $frozen = $self->serializer->serialize($data);
+	my @columns = qw/task_id result/;
+	my @values = ($self->{task_id});
+	if (ref $data eq 'HASH') {
+		push @columns, 'resulttype';
+		my $type = delete $data->{type};
+		push @values, $self->serializer->serialize($data), $type;
+	} else {
+		push @values, $self->serializer->serialize($data);
+	}
+	my $columns = join ', ', @columns;
+	my $qs = join(',', ('?') x @columns);
 	my $sql = qq{
 		INSERT INTO "$self->{database_schema}".$self->{current_table}
-			(task_id,result)
-		VALUES (?,?)
+			($columns)
+		VALUES ($qs)
 		RETURNING result_id
 	};
-	$self->insert(sql => $sql,data => [$self->{task_id},$frozen]);
+	$self->insert(sql => $sql,data => \@values);
 }
 
+
 sub fetch_result {
-	my ($self,$id) = @_;
+	my ($self,$result_id) = @_;
+	$self->{current_table} = 'result';
+	my $sql = qq{
+		SELECT *
+		FROM "$self->{database_schema}".$self->{current_table}
+		WHERE result_id=?
+	};
+	my $result = $self->select_first(sql => $sql,data => [$result_id]) || return;
+
+	my $r = $self->serializer->deserialize($result->{result});
+	$result->{result} = $r;
+	return $result;
+}
+
+
+sub fetch_first_result {
+	my ($self,$task_id) = @_;
 	$self->{current_table} = 'result';
 	my $sql = qq{
 		SELECT *
@@ -187,11 +244,12 @@ sub fetch_result {
 		WHERE task_id=?
 		ORDER BY result_id DESC
 	};
-	my $result = $self->select_first(sql => $sql,data => [$id]) || return;
+	my $result = $self->select_first(sql => $sql,data => [$task_id]) || return;
 
 	return $self->serializer->deserialize($result->{result});
 }
 
+
 sub fetch_results {
 	my ($self,$id) = @_;
 	$self->{current_table} = 'result';
@@ -203,12 +261,51 @@ sub fetch_results {
 	};
 	my $results = $self->select_all(sql => $sql,data => [$id]) || return;
 
-	return [map { $self->serializer->deserialize($_->{result}) } @{ $results } ];
+	return [map { {id => $_->{result_id}, type => $_->{resulttype}, result => $self->serializer->deserialize($_->{result}) } } @{ $results } ];
+}
+
+
+sub get_statuses {
+	my ($self) = @_;
+	$self->{current_table} = 'task';
+	my $sql = qq{
+		SELECT status
+		FROM "$self->{database_schema}".$self->{current_table}
+		GROUP BY status
+	};
+	my $stats = $self->select_all(sql => $sql) || return;
+	return $stats;
+}
+
+
+sub get_classes {
+	my ($self) = @_;
+	$self->{current_table} = 'class';
+	my $sql = qq{
+		SELECT *
+		FROM "$self->{database_schema}".$self->{current_table}
+	};
+	my $stats = $self->select_all(sql => $sql) || return;
+	return $stats;
+}
+
+
+sub get_tasks {
+	my ($self,%args) = @_;
+	$self->{current_table} = 'task';
+	my ($where_clause, @where_args) = $self->where_clause($args{where});
+	my $order_by = $self->order_by($args{order_by});
+	my $sql = qq{
+		SELECT *
+		FROM "$self->{database_schema}".$self->{current_table} t
+		JOIN "$self->{database_schema}".class c USING (class_id)
+		$where_clause
+		$order_by
+	};
+	my $tasks = $self->select_all(sql => $sql,data => \@where_args) || return;
+	return $tasks;
 }
 
-# 1. Find started tasks that have passed the time limit, most probably because 
-# of a dead worker. (status 100, modified < now - max_runtime)
-# 2. Trim status so task can be tried again
 
 sub revive_tasks {
 	my ($self,$max) = @_;
@@ -224,10 +321,6 @@ sub revive_tasks {
 	return $result;
 }
 
-# 1. Find tasks that have failed too many times (# of result rows > $self->retries
-# 2. fail them (Set status 900)
-# There's a hard limit (100) for how many tasks can be failed at one time for
-# performance resons
 
 sub fail_tasks {
 	my ($self,$retries) = @_;
@@ -255,9 +348,7 @@ sub fail_tasks {
 	return scalar @$result;
 }
 
-# 3. Find tasks that should be removed (remove_task < now)
-# - delete them
-# - log
+
 sub remove_tasks {
 	my ($self,$after) = @_;
 	return 0 unless $after;
@@ -272,6 +363,7 @@ sub remove_tasks {
 	return $result;
 }
 
+
 sub select_first {
 	my ($self, %args) = @_;
 	my $sth = $self->dbh->prepare($args{sql}) || return 0;
@@ -287,10 +379,10 @@ sub select_first {
 	return ( $r );
 }
 
+
 sub select_all {
 	my ($self, %args) = @_;
 	my $sth = $self->dbh->prepare($args{sql}) || return 0;
-
 	$self->set_bind_type($sth,$args{data} || []);
 	unless($sth->execute(@{$args{data}})) {
 		my @c = caller;
@@ -361,6 +453,22 @@ sub DESTROY {
 	return;
 }
 
+
+sub where_clause {
+	my ($self, $where) = @_;
+	my $where_clause = join(' AND ', ("$_ = ?") x keys %$where);
+	$where_clause = "WHERE $where_clause" if $where_clause;
+	return $where_clause, values %$where;
+}
+
+
+sub order_by {
+	my ($self, $order) = @_;
+	my $order_by = join(',', ("$_") x keys @$order);
+	$order_by = "ORDER BY $order_by" if $order_by;
+	return $order_by;
+}
+
 1;
 
 __END__
@@ -375,7 +483,7 @@ Job::Machine::DB
 
 =head1 VERSION
 
-version 0.22
+version 0.23
 
 =head1 NAME
 
@@ -395,52 +503,153 @@ Job::Machine::DB - Database class for Job::Machine
 	  dsn   => @dsn,
   );
 
-=head2 set_listen
+=head2 serializer
+
+Returns the serializer, default Data::Serializer
+
+=head2 listen
+
+Sets up the listener.  Quit listening to the named queues. If 'reply' is
+passed, we listen to the related reply queue instead of the task queue.
+
+Return undef immediately if no queue is provided.
 
  $self->listen( queue => 'queue_name' );
  $self->listen( queue => \@queues, reply => 1  );
 
-Sets up the listener.  Quit listening to the named queues. If 'reply' is
-passed, we unlisten to the related reply queue instead of the task queue.
-
-Return undef immediately if no queue is provided.
-
 =head2 unlisten
 
- $self->unlisten( queue => 'queue_name' );
- $self->unlisten( queue => \@queues, reply => 1  );
-
 Quit listening to the named queues. If 'reply' is passed, we unlisten
 to the related reply queue instead of the task queue.
 
 Return undef immediately if no queue is provided.
 
-=head2 notify
+ $self->unlisten( queue => 'queue_name' );
+ $self->unlisten( queue => \@queues, reply => 1  );
 
- $self->notify( queue => 'queue_name' );
- $self->notify( queue => 'queue_name', reply => 1, payload => $data  );
+=head2 notify
 
 Sends an asynchronous notification to the named queue, with an optional
 payload. If 'reply' is true, then the queue names are taken to be reply.
 
 Return undef immediately if no queue name is provided.
 
+ $self->notify( queue => 'queue_name' );
+ $self->notify( queue => 'queue_name', reply => 1, payload => $data  );
+
 =head2 get_notification
 
+Retrieve one notification, if there is one
+
+Retrievies the pending notifications.
+
  my $notifies = $self->get_notification();
 
-Retrievies the pending notifications. The return value is an arrayref where
-each row looks like this:
+The return value is an arrayref where each row looks like this:
 
  my ($name, $pid, $payload) = @$notify;
 
+=head2 set_listen
+
+Wait for a notification. The required parameter timeout tells for how long time to wait.
+
+=head2 fetch_work_task
+
+Fetch one work task from the task table
+
+=head2 insert_task
+
+Insert a row in the task table
+
+=head2 set_task_status
+
+Update the task with a new status
+
+=head2 fetch_class
+
+Fetch a class
+
+=head2 fetch_task
+
+Fetch a task
+
+=head2 insert_class
+
+Insert a row in the class table
+
+=head2 insert_result
+
+Insert a row in the result table
+
+Argument
+
+ data - either a scalar value that will be inserted as the result, or a hashref containing the type and result
+
+=head2 fetch_result
+
+Fetch a result using the result id
+
+=head2 fetch_first_result
+
+Fetch a result using the task id
+
+=head2 fetch_results
+
+Fetch all results of a given task
+
+=head2 get_statuses
+
+Fetch all distinct statuses
+
+=head2 get_classes
+
+Fetch all classes
+
+=head2 get_tasks
+
+Fetch all tasks, joined with the class for a suitable name
+
+=head2 revive_tasks
+
+	1. Find started tasks that have passed the time limit, most probably because of a dead worker. (status 100, modified < now - max_runtime)
+	2. Trim status so task can be tried again
+
+=head2 fail_tasks
+
+	1. Find tasks that have failed too many times (# of result rows > $self->retries
+	2. fail them (Set status 900)
+	There's a hard limit (100) for how many tasks can be failed at one time for
+	performance resons
+
+=head2 remove_tasks
+
+	3. Find tasks that should be removed (remove_task < now)
+	- delete them
+	- log
+
+=head2 select_first
+
+Select the first row from the given sql statement
+
+=head2 select_all
+
+Select all rows from the given sql statement
+
+=head2 where_clause
+
+Very light weight where clause builder
+
+=head2 order_by
+
+Very light weight order-by builder
+
 =head1 AUTHOR
 
 Kaare Rasmussen <kaare@cpan.org>.
 
 =head1 COPYRIGHT
 
-Copyright (C) 2009,2014, Kaare Rasmussen
+Copyright (C) 2009,2015, Kaare Rasmussen
 
 This module is free software; you can redistribute it or modify it
 under the same terms as Perl itself.
